# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import date, datetime
from typing_extensions import Literal

from .._models import BaseModel
from .shared.currency import Currency

__all__ = ["Transaction", "ForeignExchangeRate"]


class ForeignExchangeRate(BaseModel):
    base_amount: int
    """
    Amount in the lowest denomination of the `base_currency` to convert, often
    called the "sell" amount.
    """

    base_currency: Currency
    """Currency to convert, often called the "sell" currency."""

    exponent: int
    """The exponent component of the rate.

    The decimal is calculated as `value` / (10 ^ `exponent`).
    """

    rate_string: str
    """A string representation of the rate."""

    target_amount: int
    """
    Amount in the lowest denomination of the `target_currency`, often called the
    "buy" amount.
    """

    target_currency: Currency
    """Currency to convert the `base_currency` to, often called the "buy" currency."""

    value: int
    """The whole number component of the rate.

    The decimal is calculated as `value` / (10 ^ `exponent`).
    """


class Transaction(BaseModel):
    id: str

    amount: int
    """Value in specified currency's smallest unit.

    e.g. $10 would be represented as 1000.
    """

    as_of_date: Optional[date] = None
    """The date on which the transaction occurred."""

    as_of_time: Optional[str] = None
    """The time on which the transaction occurred.

    Depending on the granularity of the timestamp information received from the
    bank, it may be `null`.
    """

    as_of_timezone: Optional[str] = None
    """The timezone in which the `as_of_time` is represented.

    Can be `null` if the bank does not provide timezone info.
    """

    created_at: datetime

    currency: Currency
    """Currency that this transaction is denominated in."""

    custom_identifiers: Dict[str, str]
    """
    An object containing key-value pairs, each with a custom identifier as the key
    and a string value.
    """

    direction: str
    """Either `credit` or `debit`."""

    discarded_at: Optional[datetime] = None

    foreign_exchange_rate: Optional[ForeignExchangeRate] = None
    """Associated serialized foreign exchange rate information."""

    internal_account_id: str
    """The ID of the relevant Internal Account."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    object: str

    posted: bool
    """This field will be `true` if the transaction has posted to the account."""

    reconciled: bool
    """
    This field will be `true` if a transaction is reconciled by the Modern Treasury
    system. This means that it has transaction line items that sum up to the
    transaction's amount.
    """

    type: Literal[
        "ach",
        "au_becs",
        "bacs",
        "base",
        "book",
        "card",
        "chats",
        "check",
        "cross_border",
        "dk_nets",
        "eft",
        "ethereum",
        "hu_ics",
        "interac",
        "masav",
        "mx_ccen",
        "neft",
        "nics",
        "nz_becs",
        "pl_elixir",
        "polygon",
        "provxchange",
        "ro_sent",
        "rtp",
        "se_bankgirot",
        "sen",
        "sepa",
        "sg_giro",
        "sic",
        "signet",
        "sknbi",
        "solana",
        "wire",
        "zengin",
        "other",
    ]
    """The type of the transaction.

    Examples could be `card, `ach`, `wire`, `check`, `rtp`, `book`, or `sen`.
    """

    updated_at: datetime

    vendor_code: Optional[str] = None
    """When applicable, the bank-given code that determines the transaction's category.

    For most banks this is the BAI2/BTRS transaction code.
    """

    vendor_code_type: Optional[
        Literal[
            "bai2",
            "banking_circle",
            "bankprov",
            "bnk_dev",
            "brale",
            "bridge",
            "cleartouch",
            "column",
            "cross_river",
            "currencycloud",
            "dc_bank",
            "dwolla",
            "evolve",
            "goldman_sachs",
            "iso20022",
            "jpmc",
            "mx",
            "paypal",
            "plaid",
            "pnc",
            "rspec_vendor",
            "signet",
            "silvergate",
            "swift",
            "us_bank",
            "user",
        ]
    ] = None
    """The type of `vendor_code` being reported.

    Can be one of `bai2`, `bankprov`, `bnk_dev`, `cleartouch`, `currencycloud`,
    `cross_river`, `dc_bank`, `dwolla`, `evolve`, `goldman_sachs`, `iso20022`,
    `jpmc`, `mx`, `signet`, `silvergate`, `swift`, `us_bank`, or others.
    """

    vendor_customer_id: Optional[str] = None
    """An identifier given to this transaction by the bank, often `null`."""

    vendor_id: Optional[str] = None
    """An identifier given to this transaction by the bank."""

    details: Optional[Dict[str, str]] = None
    """
    This field contains additional information that the bank provided about the
    transaction. This is structured data. Some of the data in here might overlap
    with what is in the `vendor_description`. For example, the OBI could be a part
    of the vendor description, and it would also be included in here. The attributes
    that are passed through the details field will vary based on your banking
    partner. Currently, the following keys may be in the details object:
    `originator_name`, `originator_to_beneficiary_information`.
    """

    vendor_description: Optional[str] = None
    """
    The transaction detail text that often appears in on your bank statement and in
    your banking portal.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from .shared.transaction_direction import TransactionDirection

__all__ = [
    "VirtualAccountCreateParams",
    "AccountDetails",
    "AccountDetail",
    "LedgerAccount",
    "RoutingDetails",
    "RoutingDetail",
]


class VirtualAccountCreateParams(TypedDict, total=False):
    internal_account_id: Required[str]
    """The ID of the internal account that this virtual account is associated with."""

    name: Required[str]
    """The name of the virtual account."""

    account_details: Iterable[AccountDetail]
    """An array of account detail objects."""

    counterparty_id: str
    """The ID of the counterparty that the virtual account belongs to."""

    credit_ledger_account_id: str
    """The ID of a credit normal ledger account.

    When money leaves the virtual account, this ledger account will be credited.
    Must be accompanied by a debit_ledger_account_id if present.
    """

    debit_ledger_account_id: str
    """The ID of a debit normal ledger account.

    When money enters the virtual account, this ledger account will be debited. Must
    be accompanied by a credit_ledger_account_id if present.
    """

    description: str
    """An optional description for internal use."""

    ledger_account: LedgerAccount
    """Specifies a ledger account object that will be created with the virtual account.

    The resulting ledger account is linked to the virtual account for auto-ledgering
    IPDs.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    routing_details: Iterable[RoutingDetail]
    """An array of routing detail objects."""


class AccountDetail(TypedDict, total=False):
    account_number: Required[str]
    """The account number for the bank account."""

    account_number_type: Literal[
        "au_number",
        "base_address",
        "clabe",
        "ethereum_address",
        "hk_number",
        "iban",
        "id_number",
        "nz_number",
        "other",
        "pan",
        "polygon_address",
        "sg_number",
        "solana_address",
        "wallet_address",
    ]
    """One of `iban`, `clabe`, `wallet_address`, or `other`.

    Use `other` if the bank account number is in a generic format.
    """


AccountDetails = AccountDetail
"""This type is deprecated and will be removed in a future release.

Please use AccountDetail instead.
"""


class LedgerAccount(TypedDict, total=False):
    currency: Required[str]
    """The currency of the ledger account."""

    ledger_id: Required[str]
    """The id of the ledger that this account belongs to."""

    name: Required[str]
    """The name of the ledger account."""

    normal_balance: Required[TransactionDirection]
    """The normal balance of the ledger account."""

    currency_exponent: Optional[int]
    """The currency exponent of the ledger account."""

    description: Optional[str]
    """The description of the ledger account."""

    ledger_account_category_ids: List[str]
    """
    The array of ledger account category ids that this ledger account should be a
    child of.
    """

    ledgerable_id: str
    """
    If the ledger account links to another object in Modern Treasury, the id will be
    populated here, otherwise null.
    """

    ledgerable_type: Literal["counterparty", "external_account", "internal_account", "virtual_account"]
    """
    If the ledger account links to another object in Modern Treasury, the type will
    be populated here, otherwise null. The value is one of internal_account or
    external_account.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """


class RoutingDetail(TypedDict, total=False):
    routing_number: Required[str]
    """The routing number of the bank."""

    routing_number_type: Required[
        Literal[
            "aba",
            "au_bsb",
            "br_codigo",
            "ca_cpa",
            "chips",
            "cnaps",
            "dk_interbank_clearing_code",
            "gb_sort_code",
            "hk_interbank_clearing_code",
            "hu_interbank_clearing_code",
            "id_sknbi_code",
            "in_ifsc",
            "jp_zengin_code",
            "mx_bank_identifier",
            "my_branch_code",
            "nz_national_clearing_code",
            "pl_national_clearing_code",
            "se_bankgiro_clearing_code",
            "sg_interbank_clearing_code",
            "swift",
            "za_national_clearing_code",
        ]
    ]
    """The type of routing number.

    See https://docs.moderntreasury.com/platform/reference/routing-detail-object for
    more details.
    """

    payment_type: Optional[
        Literal[
            "ach",
            "au_becs",
            "bacs",
            "base",
            "book",
            "card",
            "chats",
            "check",
            "cross_border",
            "dk_nets",
            "eft",
            "ethereum",
            "hu_ics",
            "interac",
            "masav",
            "mx_ccen",
            "neft",
            "nics",
            "nz_becs",
            "pl_elixir",
            "polygon",
            "provxchange",
            "ro_sent",
            "rtp",
            "se_bankgirot",
            "sen",
            "sepa",
            "sg_giro",
            "sic",
            "signet",
            "sknbi",
            "solana",
            "wire",
            "zengin",
        ]
    ]
    """
    If the routing detail is to be used for a specific payment type this field will
    be populated, otherwise null.
    """


RoutingDetails = RoutingDetail
"""This type is deprecated and will be removed in a future release.

Please use RoutingDetail instead.
"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["LegalEntityComplianceDetail"]


class LegalEntityComplianceDetail(BaseModel):
    id: str

    created_at: datetime

    discarded_at: Optional[datetime] = None

    issuer: str
    """The issuer of the compliance token."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    token_expires_at: Optional[datetime] = None
    """The timestamp when the compliance token expires."""

    token_issued_at: Optional[datetime] = None
    """The timestamp when the compliance token was issued."""

    token_url: Optional[str] = None
    """The URL to the compliance token. (ex. provider portal URL)"""

    updated_at: datetime

    validated: bool
    """Whether entity corresponding to the compliance token has been validated."""

    validated_at: Optional[datetime] = None
    """The timestamp when the entity was validated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PaymentActionListParams", "CreatedAt"]


class PaymentActionListParams(TypedDict, total=False):
    actionable_id: str
    """The ID of the associated actionable object."""

    actionable_type: str
    """The type of the associated actionable object.

    One of `payment_order`, `expected_payment`.
    """

    after_cursor: Optional[str]

    created_at: CreatedAt
    """
    Filter by `created_at` using comparison operators like `gt` (>), `gte` (>=),
    `lt` (<), `lte` (<=), or `eq` (=) to filter by the created at timestamp. For
    example, `created_at[gte]=2024-01-01T00:00:00Z`
    """

    internal_account_id: str
    """The ID of one of your internal accounts."""

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    per_page: int

    status: Literal["pending", "processable", "processing", "sent", "failed", "cancelled"]
    """Filter by payment actions of a specific status."""

    type: Literal["stop", "issue"]
    """The type of payment action."""


class CreatedAt(TypedDict, total=False):
    eq: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    gt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    lt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from modern_treasury import ModernTreasury, AsyncModernTreasury

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAccountEntries:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_update(self, client: ModernTreasury) -> None:
        account_entry = client.ledger_account_settlements.account_entries.update(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert account_entry is None

    @parametrize
    def test_raw_response_update(self, client: ModernTreasury) -> None:
        response = client.ledger_account_settlements.account_entries.with_raw_response.update(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_entry = response.parse()
        assert account_entry is None

    @parametrize
    def test_streaming_response_update(self, client: ModernTreasury) -> None:
        with client.ledger_account_settlements.account_entries.with_streaming_response.update(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_entry = response.parse()
            assert account_entry is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.ledger_account_settlements.account_entries.with_raw_response.update(
                id="",
                ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            )

    @parametrize
    def test_method_delete(self, client: ModernTreasury) -> None:
        account_entry = client.ledger_account_settlements.account_entries.delete(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert account_entry is None

    @parametrize
    def test_raw_response_delete(self, client: ModernTreasury) -> None:
        response = client.ledger_account_settlements.account_entries.with_raw_response.delete(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_entry = response.parse()
        assert account_entry is None

    @parametrize
    def test_streaming_response_delete(self, client: ModernTreasury) -> None:
        with client.ledger_account_settlements.account_entries.with_streaming_response.delete(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_entry = response.parse()
            assert account_entry is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.ledger_account_settlements.account_entries.with_raw_response.delete(
                id="",
                ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            )


class TestAsyncAccountEntries:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_update(self, async_client: AsyncModernTreasury) -> None:
        account_entry = await async_client.ledger_account_settlements.account_entries.update(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert account_entry is None

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.ledger_account_settlements.account_entries.with_raw_response.update(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_entry = response.parse()
        assert account_entry is None

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.ledger_account_settlements.account_entries.with_streaming_response.update(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_entry = await response.parse()
            assert account_entry is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.ledger_account_settlements.account_entries.with_raw_response.update(
                id="",
                ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncModernTreasury) -> None:
        account_entry = await async_client.ledger_account_settlements.account_entries.delete(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert account_entry is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.ledger_account_settlements.account_entries.with_raw_response.delete(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_entry = response.parse()
        assert account_entry is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.ledger_account_settlements.account_entries.with_streaming_response.delete(
            id="id",
            ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_entry = await response.parse()
            assert account_entry is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.ledger_account_settlements.account_entries.with_raw_response.delete(
                id="",
                ledger_entry_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from datetime import date
from typing_extensions import Literal

import httpx

from .. import _legacy_response
from ..types import (
    legal_entity_list_params,
    legal_entity_create_params,
    legal_entity_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.legal_entity import LegalEntity
from ..types.bank_settings_param import BankSettingsParam
from ..types.wealth_and_employment_details_param import WealthAndEmploymentDetailsParam
from ..types.shared_params.legal_entity_compliance_detail import LegalEntityComplianceDetail
from ..types.shared_params.legal_entity_industry_classification import LegalEntityIndustryClassification

__all__ = ["LegalEntities", "AsyncLegalEntities"]


class LegalEntities(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LegalEntitiesWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return LegalEntitiesWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LegalEntitiesWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return LegalEntitiesWithStreamingResponse(self)

    def create(
        self,
        *,
        legal_entity_type: Literal["business", "individual"],
        addresses: Iterable[legal_entity_create_params.Address] | NotGiven = NOT_GIVEN,
        bank_settings: Optional[BankSettingsParam] | NotGiven = NOT_GIVEN,
        business_name: Optional[str] | NotGiven = NOT_GIVEN,
        citizenship_country: Optional[str] | NotGiven = NOT_GIVEN,
        compliance_details: Optional[LegalEntityComplianceDetail] | NotGiven = NOT_GIVEN,
        date_formed: Union[str, date, None] | NotGiven = NOT_GIVEN,
        date_of_birth: Union[str, date, None] | NotGiven = NOT_GIVEN,
        doing_business_as_names: List[str] | NotGiven = NOT_GIVEN,
        email: Optional[str] | NotGiven = NOT_GIVEN,
        first_name: Optional[str] | NotGiven = NOT_GIVEN,
        identifications: Iterable[legal_entity_create_params.Identification] | NotGiven = NOT_GIVEN,
        industry_classifications: Iterable[LegalEntityIndustryClassification] | NotGiven = NOT_GIVEN,
        last_name: Optional[str] | NotGiven = NOT_GIVEN,
        legal_entity_associations: Optional[Iterable[legal_entity_create_params.LegalEntityAssociation]]
        | NotGiven = NOT_GIVEN,
        legal_structure: Optional[
            Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        middle_name: Optional[str] | NotGiven = NOT_GIVEN,
        phone_numbers: Iterable[legal_entity_create_params.PhoneNumber] | NotGiven = NOT_GIVEN,
        politically_exposed_person: Optional[bool] | NotGiven = NOT_GIVEN,
        preferred_name: Optional[str] | NotGiven = NOT_GIVEN,
        prefix: Optional[str] | NotGiven = NOT_GIVEN,
        risk_rating: Optional[Literal["low", "medium", "high"]] | NotGiven = NOT_GIVEN,
        suffix: Optional[str] | NotGiven = NOT_GIVEN,
        wealth_and_employment_details: Optional[WealthAndEmploymentDetailsParam] | NotGiven = NOT_GIVEN,
        website: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LegalEntity:
        """
        create legal_entity

        Args:
          legal_entity_type: The type of legal entity.

          addresses: A list of addresses for the entity.

          business_name: The business's legal business name.

          citizenship_country: The country of citizenship for an individual.

          date_formed: A business's formation date (YYYY-MM-DD).

          date_of_birth: An individual's date of birth (YYYY-MM-DD).

          email: The entity's primary email.

          first_name: An individual's first name.

          identifications: A list of identifications for the legal entity.

          industry_classifications: A list of industry classifications for the legal entity.

          last_name: An individual's last name.

          legal_entity_associations: The legal entity associations and its child legal entities.

          legal_structure: The business's legal structure.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          middle_name: An individual's middle name.

          politically_exposed_person: Whether the individual is a politically exposed person.

          preferred_name: An individual's preferred name.

          prefix: An individual's prefix.

          risk_rating: The risk rating of the legal entity. One of low, medium, high.

          suffix: An individual's suffix.

          website: The entity's primary website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/legal_entities",
            body=maybe_transform(
                {
                    "legal_entity_type": legal_entity_type,
                    "addresses": addresses,
                    "bank_settings": bank_settings,
                    "business_name": business_name,
                    "citizenship_country": citizenship_country,
                    "compliance_details": compliance_details,
                    "date_formed": date_formed,
                    "date_of_birth": date_of_birth,
                    "doing_business_as_names": doing_business_as_names,
                    "email": email,
                    "first_name": first_name,
                    "identifications": identifications,
                    "industry_classifications": industry_classifications,
                    "last_name": last_name,
                    "legal_entity_associations": legal_entity_associations,
                    "legal_structure": legal_structure,
                    "metadata": metadata,
                    "middle_name": middle_name,
                    "phone_numbers": phone_numbers,
                    "politically_exposed_person": politically_exposed_person,
                    "preferred_name": preferred_name,
                    "prefix": prefix,
                    "risk_rating": risk_rating,
                    "suffix": suffix,
                    "wealth_and_employment_details": wealth_and_employment_details,
                    "website": website,
                },
                legal_entity_create_params.LegalEntityCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LegalEntity,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LegalEntity:
        """
        Get details on a single legal entity.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/legal_entities/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LegalEntity,
        )

    def update(
        self,
        id: str,
        *,
        addresses: Iterable[legal_entity_update_params.Address] | NotGiven = NOT_GIVEN,
        bank_settings: Optional[BankSettingsParam] | NotGiven = NOT_GIVEN,
        business_name: Optional[str] | NotGiven = NOT_GIVEN,
        citizenship_country: Optional[str] | NotGiven = NOT_GIVEN,
        compliance_details: Optional[LegalEntityComplianceDetail] | NotGiven = NOT_GIVEN,
        date_formed: Union[str, date, None] | NotGiven = NOT_GIVEN,
        date_of_birth: Union[str, date, None] | NotGiven = NOT_GIVEN,
        doing_business_as_names: List[str] | NotGiven = NOT_GIVEN,
        email: Optional[str] | NotGiven = NOT_GIVEN,
        first_name: Optional[str] | NotGiven = NOT_GIVEN,
        identifications: Iterable[legal_entity_update_params.Identification] | NotGiven = NOT_GIVEN,
        industry_classifications: Iterable[LegalEntityIndustryClassification] | NotGiven = NOT_GIVEN,
        last_name: Optional[str] | NotGiven = NOT_GIVEN,
        legal_structure: Optional[
            Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        middle_name: Optional[str] | NotGiven = NOT_GIVEN,
        phone_numbers: Iterable[legal_entity_update_params.PhoneNumber] | NotGiven = NOT_GIVEN,
        politically_exposed_person: Optional[bool] | NotGiven = NOT_GIVEN,
        preferred_name: Optional[str] | NotGiven = NOT_GIVEN,
        prefix: Optional[str] | NotGiven = NOT_GIVEN,
        risk_rating: Optional[Literal["low", "medium", "high"]] | NotGiven = NOT_GIVEN,
        suffix: Optional[str] | NotGiven = NOT_GIVEN,
        wealth_and_employment_details: Optional[WealthAndEmploymentDetailsParam] | NotGiven = NOT_GIVEN,
        website: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LegalEntity:
        """
        Update a legal entity.

        Args:
          addresses: A list of addresses for the entity.

          business_name: The business's legal business name.

          citizenship_country: The country of citizenship for an individual.

          date_formed: A business's formation date (YYYY-MM-DD).

          date_of_birth: An individual's date of birth (YYYY-MM-DD).

          email: The entity's primary email.

          first_name: An individual's first name.

          identifications: A list of identifications for the legal entity.

          industry_classifications: A list of industry classifications for the legal entity.

          last_name: An individual's last name.

          legal_structure: The business's legal structure.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          middle_name: An individual's middle name.

          politically_exposed_person: Whether the individual is a politically exposed person.

          preferred_name: An individual's preferred name.

          prefix: An individual's prefix.

          risk_rating: The risk rating of the legal entity. One of low, medium, high.

          suffix: An individual's suffix.

          website: The entity's primary website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/legal_entities/{id}",
            body=maybe_transform(
                {
                    "addresses": addresses,
                    "bank_settings": bank_settings,
                    "business_name": business_name,
                    "citizenship_country": citizenship_country,
                    "compliance_details": compliance_details,
                    "date_formed": date_formed,
                    "date_of_birth": date_of_birth,
                    "doing_business_as_names": doing_business_as_names,
                    "email": email,
                    "first_name": first_name,
                    "identifications": identifications,
                    "industry_classifications": industry_classifications,
                    "last_name": last_name,
                    "legal_structure": legal_structure,
                    "metadata": metadata,
                    "middle_name": middle_name,
                    "phone_numbers": phone_numbers,
                    "politically_exposed_person": politically_exposed_person,
                    "preferred_name": preferred_name,
                    "prefix": prefix,
                    "risk_rating": risk_rating,
                    "suffix": suffix,
                    "wealth_and_employment_details": wealth_and_employment_details,
                    "website": website,
                },
                legal_entity_update_params.LegalEntityUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LegalEntity,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        legal_entity_type: Literal["business", "individual"] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        show_deleted: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPage[LegalEntity]:
        """
        Get a list of all legal entities.

        Args:
          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/legal_entities",
            page=SyncPage[LegalEntity],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "legal_entity_type": legal_entity_type,
                        "metadata": metadata,
                        "per_page": per_page,
                        "show_deleted": show_deleted,
                    },
                    legal_entity_list_params.LegalEntityListParams,
                ),
            ),
            model=LegalEntity,
        )


class AsyncLegalEntities(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLegalEntitiesWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLegalEntitiesWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLegalEntitiesWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncLegalEntitiesWithStreamingResponse(self)

    async def create(
        self,
        *,
        legal_entity_type: Literal["business", "individual"],
        addresses: Iterable[legal_entity_create_params.Address] | NotGiven = NOT_GIVEN,
        bank_settings: Optional[BankSettingsParam] | NotGiven = NOT_GIVEN,
        business_name: Optional[str] | NotGiven = NOT_GIVEN,
        citizenship_country: Optional[str] | NotGiven = NOT_GIVEN,
        compliance_details: Optional[LegalEntityComplianceDetail] | NotGiven = NOT_GIVEN,
        date_formed: Union[str, date, None] | NotGiven = NOT_GIVEN,
        date_of_birth: Union[str, date, None] | NotGiven = NOT_GIVEN,
        doing_business_as_names: List[str] | NotGiven = NOT_GIVEN,
        email: Optional[str] | NotGiven = NOT_GIVEN,
        first_name: Optional[str] | NotGiven = NOT_GIVEN,
        identifications: Iterable[legal_entity_create_params.Identification] | NotGiven = NOT_GIVEN,
        industry_classifications: Iterable[LegalEntityIndustryClassification] | NotGiven = NOT_GIVEN,
        last_name: Optional[str] | NotGiven = NOT_GIVEN,
        legal_entity_associations: Optional[Iterable[legal_entity_create_params.LegalEntityAssociation]]
        | NotGiven = NOT_GIVEN,
        legal_structure: Optional[
            Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        middle_name: Optional[str] | NotGiven = NOT_GIVEN,
        phone_numbers: Iterable[legal_entity_create_params.PhoneNumber] | NotGiven = NOT_GIVEN,
        politically_exposed_person: Optional[bool] | NotGiven = NOT_GIVEN,
        preferred_name: Optional[str] | NotGiven = NOT_GIVEN,
        prefix: Optional[str] | NotGiven = NOT_GIVEN,
        risk_rating: Optional[Literal["low", "medium", "high"]] | NotGiven = NOT_GIVEN,
        suffix: Optional[str] | NotGiven = NOT_GIVEN,
        wealth_and_employment_details: Optional[WealthAndEmploymentDetailsParam] | NotGiven = NOT_GIVEN,
        website: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LegalEntity:
        """
        create legal_entity

        Args:
          legal_entity_type: The type of legal entity.

          addresses: A list of addresses for the entity.

          business_name: The business's legal business name.

          citizenship_country: The country of citizenship for an individual.

          date_formed: A business's formation date (YYYY-MM-DD).

          date_of_birth: An individual's date of birth (YYYY-MM-DD).

          email: The entity's primary email.

          first_name: An individual's first name.

          identifications: A list of identifications for the legal entity.

          industry_classifications: A list of industry classifications for the legal entity.

          last_name: An individual's last name.

          legal_entity_associations: The legal entity associations and its child legal entities.

          legal_structure: The business's legal structure.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          middle_name: An individual's middle name.

          politically_exposed_person: Whether the individual is a politically exposed person.

          preferred_name: An individual's preferred name.

          prefix: An individual's prefix.

          risk_rating: The risk rating of the legal entity. One of low, medium, high.

          suffix: An individual's suffix.

          website: The entity's primary website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/legal_entities",
            body=await async_maybe_transform(
                {
                    "legal_entity_type": legal_entity_type,
                    "addresses": addresses,
                    "bank_settings": bank_settings,
                    "business_name": business_name,
                    "citizenship_country": citizenship_country,
                    "compliance_details": compliance_details,
                    "date_formed": date_formed,
                    "date_of_birth": date_of_birth,
                    "doing_business_as_names": doing_business_as_names,
                    "email": email,
                    "first_name": first_name,
                    "identifications": identifications,
                    "industry_classifications": industry_classifications,
                    "last_name": last_name,
                    "legal_entity_associations": legal_entity_associations,
                    "legal_structure": legal_structure,
                    "metadata": metadata,
                    "middle_name": middle_name,
                    "phone_numbers": phone_numbers,
                    "politically_exposed_person": politically_exposed_person,
                    "preferred_name": preferred_name,
                    "prefix": prefix,
                    "risk_rating": risk_rating,
                    "suffix": suffix,
                    "wealth_and_employment_details": wealth_and_employment_details,
                    "website": website,
                },
                legal_entity_create_params.LegalEntityCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LegalEntity,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LegalEntity:
        """
        Get details on a single legal entity.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/legal_entities/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LegalEntity,
        )

    async def update(
        self,
        id: str,
        *,
        addresses: Iterable[legal_entity_update_params.Address] | NotGiven = NOT_GIVEN,
        bank_settings: Optional[BankSettingsParam] | NotGiven = NOT_GIVEN,
        business_name: Optional[str] | NotGiven = NOT_GIVEN,
        citizenship_country: Optional[str] | NotGiven = NOT_GIVEN,
        compliance_details: Optional[LegalEntityComplianceDetail] | NotGiven = NOT_GIVEN,
        date_formed: Union[str, date, None] | NotGiven = NOT_GIVEN,
        date_of_birth: Union[str, date, None] | NotGiven = NOT_GIVEN,
        doing_business_as_names: List[str] | NotGiven = NOT_GIVEN,
        email: Optional[str] | NotGiven = NOT_GIVEN,
        first_name: Optional[str] | NotGiven = NOT_GIVEN,
        identifications: Iterable[legal_entity_update_params.Identification] | NotGiven = NOT_GIVEN,
        industry_classifications: Iterable[LegalEntityIndustryClassification] | NotGiven = NOT_GIVEN,
        last_name: Optional[str] | NotGiven = NOT_GIVEN,
        legal_structure: Optional[
            Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        middle_name: Optional[str] | NotGiven = NOT_GIVEN,
        phone_numbers: Iterable[legal_entity_update_params.PhoneNumber] | NotGiven = NOT_GIVEN,
        politically_exposed_person: Optional[bool] | NotGiven = NOT_GIVEN,
        preferred_name: Optional[str] | NotGiven = NOT_GIVEN,
        prefix: Optional[str] | NotGiven = NOT_GIVEN,
        risk_rating: Optional[Literal["low", "medium", "high"]] | NotGiven = NOT_GIVEN,
        suffix: Optional[str] | NotGiven = NOT_GIVEN,
        wealth_and_employment_details: Optional[WealthAndEmploymentDetailsParam] | NotGiven = NOT_GIVEN,
        website: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LegalEntity:
        """
        Update a legal entity.

        Args:
          addresses: A list of addresses for the entity.

          business_name: The business's legal business name.

          citizenship_country: The country of citizenship for an individual.

          date_formed: A business's formation date (YYYY-MM-DD).

          date_of_birth: An individual's date of birth (YYYY-MM-DD).

          email: The entity's primary email.

          first_name: An individual's first name.

          identifications: A list of identifications for the legal entity.

          industry_classifications: A list of industry classifications for the legal entity.

          last_name: An individual's last name.

          legal_structure: The business's legal structure.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          middle_name: An individual's middle name.

          politically_exposed_person: Whether the individual is a politically exposed person.

          preferred_name: An individual's preferred name.

          prefix: An individual's prefix.

          risk_rating: The risk rating of the legal entity. One of low, medium, high.

          suffix: An individual's suffix.

          website: The entity's primary website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/legal_entities/{id}",
            body=await async_maybe_transform(
                {
                    "addresses": addresses,
                    "bank_settings": bank_settings,
                    "business_name": business_name,
                    "citizenship_country": citizenship_country,
                    "compliance_details": compliance_details,
                    "date_formed": date_formed,
                    "date_of_birth": date_of_birth,
                    "doing_business_as_names": doing_business_as_names,
                    "email": email,
                    "first_name": first_name,
                    "identifications": identifications,
                    "industry_classifications": industry_classifications,
                    "last_name": last_name,
                    "legal_structure": legal_structure,
                    "metadata": metadata,
                    "middle_name": middle_name,
                    "phone_numbers": phone_numbers,
                    "politically_exposed_person": politically_exposed_person,
                    "preferred_name": preferred_name,
                    "prefix": prefix,
                    "risk_rating": risk_rating,
                    "suffix": suffix,
                    "wealth_and_employment_details": wealth_and_employment_details,
                    "website": website,
                },
                legal_entity_update_params.LegalEntityUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LegalEntity,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        legal_entity_type: Literal["business", "individual"] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        show_deleted: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[LegalEntity, AsyncPage[LegalEntity]]:
        """
        Get a list of all legal entities.

        Args:
          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/legal_entities",
            page=AsyncPage[LegalEntity],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "legal_entity_type": legal_entity_type,
                        "metadata": metadata,
                        "per_page": per_page,
                        "show_deleted": show_deleted,
                    },
                    legal_entity_list_params.LegalEntityListParams,
                ),
            ),
            model=LegalEntity,
        )


class LegalEntitiesWithRawResponse:
    def __init__(self, legal_entities: LegalEntities) -> None:
        self._legal_entities = legal_entities

        self.create = _legacy_response.to_raw_response_wrapper(
            legal_entities.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            legal_entities.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            legal_entities.update,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            legal_entities.list,
        )


class AsyncLegalEntitiesWithRawResponse:
    def __init__(self, legal_entities: AsyncLegalEntities) -> None:
        self._legal_entities = legal_entities

        self.create = _legacy_response.async_to_raw_response_wrapper(
            legal_entities.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            legal_entities.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            legal_entities.update,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            legal_entities.list,
        )


class LegalEntitiesWithStreamingResponse:
    def __init__(self, legal_entities: LegalEntities) -> None:
        self._legal_entities = legal_entities

        self.create = to_streamed_response_wrapper(
            legal_entities.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            legal_entities.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            legal_entities.update,
        )
        self.list = to_streamed_response_wrapper(
            legal_entities.list,
        )


class AsyncLegalEntitiesWithStreamingResponse:
    def __init__(self, legal_entities: AsyncLegalEntities) -> None:
        self._legal_entities = legal_entities

        self.create = async_to_streamed_response_wrapper(
            legal_entities.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            legal_entities.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            legal_entities.update,
        )
        self.list = async_to_streamed_response_wrapper(
            legal_entities.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["LegalEntityComplianceDetail"]


class LegalEntityComplianceDetail(TypedDict, total=False):
    id: Required[str]

    created_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    discarded_at: Required[Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]]

    issuer: Required[str]
    """The issuer of the compliance token."""

    live_mode: Required[bool]
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: Required[str]

    token_expires_at: Required[Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]]
    """The timestamp when the compliance token expires."""

    token_issued_at: Required[Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]]
    """The timestamp when the compliance token was issued."""

    token_url: Required[Optional[str]]
    """The URL to the compliance token. (ex. provider portal URL)"""

    updated_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    validated: Required[bool]
    """Whether entity corresponding to the compliance token has been validated."""

    validated_at: Required[Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]]
    """The timestamp when the entity was validated."""



#!/usr/bin/env -S poetry run python

from datetime import datetime

from modern_treasury import ModernTreasury

client = ModernTreasury()

now = datetime.now()

# datetime responses will always be instances of `datetime`
account = client.internal_accounts.list().items[0]
assert isinstance(account.created_at, datetime)
assert account.created_at.tzname() == "UTC"

dt = datetime.fromisoformat("2023-03-01T12:40:40+00:00")

# both `datetime` instances or datetime strings can be passed as a request param
page = client.counterparties.list(created_at_upper_bound=dt, per_page=1)
assert len(page.items) == 1

page = client.counterparties.list(created_at_upper_bound=dt.isoformat(), per_page=1)
assert len(page.items) == 1



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal, TypeAlias

__all__ = ["ExpectedPaymentType"]

ExpectedPaymentType: TypeAlias = Optional[
    Literal[
        "ach",
        "au_becs",
        "bacs",
        "base",
        "book",
        "card",
        "chats",
        "check",
        "cross_border",
        "dk_nets",
        "eft",
        "ethereum",
        "hu_ics",
        "interac",
        "masav",
        "mx_ccen",
        "neft",
        "nics",
        "nz_becs",
        "pl_elixir",
        "polygon",
        "provxchange",
        "ro_sent",
        "rtp",
        "se_bankgirot",
        "sen",
        "sepa",
        "sg_giro",
        "sic",
        "signet",
        "sknbi",
        "solana",
        "wire",
        "zengin",
    ]
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["LedgerEntryRetrieveParams"]


class LedgerEntryRetrieveParams(TypedDict, total=False):
    show_balances: bool
    """If true, response will include the balances attached to the ledger entry.

    If there is no balance available, null will be returned instead.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
import base64
from typing import TYPE_CHECKING, Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions, _legacy_response
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Body,
    Omit,
    Query,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, ModernTreasuryError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
    make_request_options,
)
from .types.ping_response import PingResponse

if TYPE_CHECKING:
    from .resources import (
        events,
        ledgers,
        returns,
        invoices,
        webhooks,
        documents,
        line_items,
        connections,
        paper_items,
        validations,
        bulk_results,
        transactions,
        bulk_requests,
        payment_flows,
        counterparties,
        ledger_entries,
        legal_entities,
        payment_orders,
        account_details,
        ledger_accounts,
        payment_actions,
        routing_details,
        virtual_accounts,
        expected_payments,
        external_accounts,
        internal_accounts,
        payment_references,
        ledger_transactions,
        foreign_exchange_quotes,
        account_collection_flows,
        incoming_payment_details,
        connection_legal_entities,
        ledger_account_categories,
        ledger_account_statements,
        legal_entity_associations,
        ledger_account_settlements,
        ledger_account_balance_monitors,
    )
    from .resources.events import Events, AsyncEvents
    from .resources.ledgers import Ledgers, AsyncLedgers
    from .resources.returns import Returns, AsyncReturns
    from .resources.documents import Documents, AsyncDocuments
    from .resources.line_items import LineItems, AsyncLineItems
    from .resources.connections import Connections, AsyncConnections
    from .resources.paper_items import PaperItems, AsyncPaperItems
    from .resources.validations import Validations, AsyncValidations
    from .resources.bulk_results import BulkResults, AsyncBulkResults
    from .resources.bulk_requests import BulkRequests, AsyncBulkRequests
    from .resources.payment_flows import PaymentFlows, AsyncPaymentFlows
    from .resources.counterparties import Counterparties, AsyncCounterparties
    from .resources.ledger_entries import LedgerEntries, AsyncLedgerEntries
    from .resources.legal_entities import LegalEntities, AsyncLegalEntities
    from .resources.account_details import AccountDetails, AsyncAccountDetails
    from .resources.ledger_accounts import LedgerAccounts, AsyncLedgerAccounts
    from .resources.payment_actions import PaymentActions, AsyncPaymentActions
    from .resources.routing_details import RoutingDetails, AsyncRoutingDetails
    from .resources.virtual_accounts import VirtualAccounts, AsyncVirtualAccounts
    from .resources.expected_payments import ExpectedPayments, AsyncExpectedPayments
    from .resources.external_accounts import ExternalAccounts, AsyncExternalAccounts
    from .resources.invoices.invoices import Invoices, AsyncInvoices
    from .resources.payment_references import PaymentReferences, AsyncPaymentReferences
    from .resources.foreign_exchange_quotes import ForeignExchangeQuotes, AsyncForeignExchangeQuotes
    from .resources.account_collection_flows import AccountCollectionFlows, AsyncAccountCollectionFlows
    from .resources.incoming_payment_details import IncomingPaymentDetails, AsyncIncomingPaymentDetails
    from .resources.connection_legal_entities import ConnectionLegalEntities, AsyncConnectionLegalEntities
    from .resources.ledger_account_categories import LedgerAccountCategories, AsyncLedgerAccountCategories
    from .resources.ledger_account_statements import LedgerAccountStatements, AsyncLedgerAccountStatements
    from .resources.legal_entity_associations import LegalEntityAssociations, AsyncLegalEntityAssociations
    from .resources.transactions.transactions import Transactions, AsyncTransactions
    from .resources.payment_orders.payment_orders import PaymentOrders, AsyncPaymentOrders
    from .resources.ledger_account_balance_monitors import (
        LedgerAccountBalanceMonitors,
        AsyncLedgerAccountBalanceMonitors,
    )
    from .resources.internal_accounts.internal_accounts import InternalAccounts, AsyncInternalAccounts
    from .resources.ledger_transactions.ledger_transactions import LedgerTransactions, AsyncLedgerTransactions
    from .resources.ledger_account_settlements.ledger_account_settlements import (
        LedgerAccountSettlements,
        AsyncLedgerAccountSettlements,
    )

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "ModernTreasury",
    "AsyncModernTreasury",
    "Client",
    "AsyncClient",
]


class ModernTreasury(SyncAPIClient):
    # client options
    api_key: str
    organization_id: str
    webhook_key: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        organization_id: str | None = None,
        webhook_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous ModernTreasury client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `MODERN_TREASURY_API_KEY`
        - `organization_id` from `MODERN_TREASURY_ORGANIZATION_ID`
        - `webhook_key` from `MODERN_TREASURY_WEBHOOK_KEY`
        """
        if api_key is None:
            api_key = os.environ.get("MODERN_TREASURY_API_KEY")
        if api_key is None:
            raise ModernTreasuryError(
                "The api_key client option must be set either by passing api_key to the client or by setting the MODERN_TREASURY_API_KEY environment variable"
            )
        self.api_key = api_key

        if organization_id is None:
            organization_id = os.environ.get("MODERN_TREASURY_ORGANIZATION_ID")
        if organization_id is None:
            raise ModernTreasuryError(
                "The organization_id client option must be set either by passing organization_id to the client or by setting the MODERN_TREASURY_ORGANIZATION_ID environment variable"
            )
        self.organization_id = organization_id

        if webhook_key is None:
            webhook_key = os.environ.get("MODERN_TREASURY_WEBHOOK_KEY")
        self.webhook_key = webhook_key

        if base_url is None:
            base_url = os.environ.get("MODERN_TREASURY_BASE_URL")
        if base_url is None:
            base_url = f"https://app.moderntreasury.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._idempotency_header = "Idempotency-Key"

    @cached_property
    def connections(self) -> Connections:
        from .resources.connections import Connections

        return Connections(self)

    @cached_property
    def counterparties(self) -> Counterparties:
        from .resources.counterparties import Counterparties

        return Counterparties(self)

    @cached_property
    def events(self) -> Events:
        from .resources.events import Events

        return Events(self)

    @cached_property
    def webhooks(self) -> webhooks.Webhooks:
        from .resources.webhooks import Webhooks

        return Webhooks(self)

    @cached_property
    def expected_payments(self) -> ExpectedPayments:
        from .resources.expected_payments import ExpectedPayments

        return ExpectedPayments(self)

    @cached_property
    def external_accounts(self) -> ExternalAccounts:
        from .resources.external_accounts import ExternalAccounts

        return ExternalAccounts(self)

    @cached_property
    def incoming_payment_details(self) -> IncomingPaymentDetails:
        from .resources.incoming_payment_details import IncomingPaymentDetails

        return IncomingPaymentDetails(self)

    @cached_property
    def invoices(self) -> Invoices:
        from .resources.invoices import Invoices

        return Invoices(self)

    @cached_property
    def documents(self) -> Documents:
        from .resources.documents import Documents

        return Documents(self)

    @cached_property
    def account_collection_flows(self) -> AccountCollectionFlows:
        from .resources.account_collection_flows import AccountCollectionFlows

        return AccountCollectionFlows(self)

    @cached_property
    def account_details(self) -> AccountDetails:
        from .resources.account_details import AccountDetails

        return AccountDetails(self)

    @cached_property
    def routing_details(self) -> RoutingDetails:
        from .resources.routing_details import RoutingDetails

        return RoutingDetails(self)

    @cached_property
    def internal_accounts(self) -> InternalAccounts:
        from .resources.internal_accounts import InternalAccounts

        return InternalAccounts(self)

    @cached_property
    def ledgers(self) -> Ledgers:
        from .resources.ledgers import Ledgers

        return Ledgers(self)

    @cached_property
    def ledger_account_categories(self) -> LedgerAccountCategories:
        from .resources.ledger_account_categories import LedgerAccountCategories

        return LedgerAccountCategories(self)

    @cached_property
    def ledger_accounts(self) -> LedgerAccounts:
        from .resources.ledger_accounts import LedgerAccounts

        return LedgerAccounts(self)

    @cached_property
    def ledger_account_balance_monitors(self) -> LedgerAccountBalanceMonitors:
        from .resources.ledger_account_balance_monitors import LedgerAccountBalanceMonitors

        return LedgerAccountBalanceMonitors(self)

    @cached_property
    def ledger_account_statements(self) -> LedgerAccountStatements:
        from .resources.ledger_account_statements import LedgerAccountStatements

        return LedgerAccountStatements(self)

    @cached_property
    def ledger_entries(self) -> LedgerEntries:
        from .resources.ledger_entries import LedgerEntries

        return LedgerEntries(self)

    @cached_property
    def ledger_transactions(self) -> LedgerTransactions:
        from .resources.ledger_transactions import LedgerTransactions

        return LedgerTransactions(self)

    @cached_property
    def line_items(self) -> LineItems:
        from .resources.line_items import LineItems

        return LineItems(self)

    @cached_property
    def payment_flows(self) -> PaymentFlows:
        from .resources.payment_flows import PaymentFlows

        return PaymentFlows(self)

    @cached_property
    def payment_orders(self) -> PaymentOrders:
        from .resources.payment_orders import PaymentOrders

        return PaymentOrders(self)

    @cached_property
    def payment_references(self) -> PaymentReferences:
        from .resources.payment_references import PaymentReferences

        return PaymentReferences(self)

    @cached_property
    def returns(self) -> Returns:
        from .resources.returns import Returns

        return Returns(self)

    @cached_property
    def transactions(self) -> Transactions:
        from .resources.transactions import Transactions

        return Transactions(self)

    @cached_property
    def validations(self) -> Validations:
        from .resources.validations import Validations

        return Validations(self)

    @cached_property
    def paper_items(self) -> PaperItems:
        from .resources.paper_items import PaperItems

        return PaperItems(self)

    @cached_property
    def virtual_accounts(self) -> VirtualAccounts:
        from .resources.virtual_accounts import VirtualAccounts

        return VirtualAccounts(self)

    @cached_property
    def bulk_requests(self) -> BulkRequests:
        from .resources.bulk_requests import BulkRequests

        return BulkRequests(self)

    @cached_property
    def bulk_results(self) -> BulkResults:
        from .resources.bulk_results import BulkResults

        return BulkResults(self)

    @cached_property
    def ledger_account_settlements(self) -> LedgerAccountSettlements:
        from .resources.ledger_account_settlements import LedgerAccountSettlements

        return LedgerAccountSettlements(self)

    @cached_property
    def foreign_exchange_quotes(self) -> ForeignExchangeQuotes:
        from .resources.foreign_exchange_quotes import ForeignExchangeQuotes

        return ForeignExchangeQuotes(self)

    @cached_property
    def connection_legal_entities(self) -> ConnectionLegalEntities:
        from .resources.connection_legal_entities import ConnectionLegalEntities

        return ConnectionLegalEntities(self)

    @cached_property
    def legal_entities(self) -> LegalEntities:
        from .resources.legal_entities import LegalEntities

        return LegalEntities(self)

    @cached_property
    def legal_entity_associations(self) -> LegalEntityAssociations:
        from .resources.legal_entity_associations import LegalEntityAssociations

        return LegalEntityAssociations(self)

    @cached_property
    def payment_actions(self) -> PaymentActions:
        from .resources.payment_actions import PaymentActions

        return PaymentActions(self)

    @cached_property
    def with_raw_response(self) -> ModernTreasuryWithRawResponse:
        return ModernTreasuryWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModernTreasuryWithStreamedResponse:
        return ModernTreasuryWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="brackets")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        credentials = f"{self.organization_id}:{self.api_key}".encode("ascii")
        header = f"Basic {base64.b64encode(credentials).decode('ascii')}"
        return {"Authorization": header}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        organization_id: str | None = None,
        webhook_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            organization_id=organization_id or self.organization_id,
            webhook_key=webhook_key or self.webhook_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def ping(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PingResponse:
        """
        A test endpoint often used to confirm credentials and headers are being passed
        in correctly.
        """
        return self.get(
            "/api/ping",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PingResponse,
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncModernTreasury(AsyncAPIClient):
    # client options
    api_key: str
    organization_id: str
    webhook_key: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        organization_id: str | None = None,
        webhook_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncModernTreasury client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `MODERN_TREASURY_API_KEY`
        - `organization_id` from `MODERN_TREASURY_ORGANIZATION_ID`
        - `webhook_key` from `MODERN_TREASURY_WEBHOOK_KEY`
        """
        if api_key is None:
            api_key = os.environ.get("MODERN_TREASURY_API_KEY")
        if api_key is None:
            raise ModernTreasuryError(
                "The api_key client option must be set either by passing api_key to the client or by setting the MODERN_TREASURY_API_KEY environment variable"
            )
        self.api_key = api_key

        if organization_id is None:
            organization_id = os.environ.get("MODERN_TREASURY_ORGANIZATION_ID")
        if organization_id is None:
            raise ModernTreasuryError(
                "The organization_id client option must be set either by passing organization_id to the client or by setting the MODERN_TREASURY_ORGANIZATION_ID environment variable"
            )
        self.organization_id = organization_id

        if webhook_key is None:
            webhook_key = os.environ.get("MODERN_TREASURY_WEBHOOK_KEY")
        self.webhook_key = webhook_key

        if base_url is None:
            base_url = os.environ.get("MODERN_TREASURY_BASE_URL")
        if base_url is None:
            base_url = f"https://app.moderntreasury.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._idempotency_header = "Idempotency-Key"

    @cached_property
    def connections(self) -> AsyncConnections:
        from .resources.connections import AsyncConnections

        return AsyncConnections(self)

    @cached_property
    def counterparties(self) -> AsyncCounterparties:
        from .resources.counterparties import AsyncCounterparties

        return AsyncCounterparties(self)

    @cached_property
    def events(self) -> AsyncEvents:
        from .resources.events import AsyncEvents

        return AsyncEvents(self)

    @cached_property
    def webhooks(self) -> webhooks.AsyncWebhooks:
        from .resources.webhooks import AsyncWebhooks

        return AsyncWebhooks(self)

    @cached_property
    def expected_payments(self) -> AsyncExpectedPayments:
        from .resources.expected_payments import AsyncExpectedPayments

        return AsyncExpectedPayments(self)

    @cached_property
    def external_accounts(self) -> AsyncExternalAccounts:
        from .resources.external_accounts import AsyncExternalAccounts

        return AsyncExternalAccounts(self)

    @cached_property
    def incoming_payment_details(self) -> AsyncIncomingPaymentDetails:
        from .resources.incoming_payment_details import AsyncIncomingPaymentDetails

        return AsyncIncomingPaymentDetails(self)

    @cached_property
    def invoices(self) -> AsyncInvoices:
        from .resources.invoices import AsyncInvoices

        return AsyncInvoices(self)

    @cached_property
    def documents(self) -> AsyncDocuments:
        from .resources.documents import AsyncDocuments

        return AsyncDocuments(self)

    @cached_property
    def account_collection_flows(self) -> AsyncAccountCollectionFlows:
        from .resources.account_collection_flows import AsyncAccountCollectionFlows

        return AsyncAccountCollectionFlows(self)

    @cached_property
    def account_details(self) -> AsyncAccountDetails:
        from .resources.account_details import AsyncAccountDetails

        return AsyncAccountDetails(self)

    @cached_property
    def routing_details(self) -> AsyncRoutingDetails:
        from .resources.routing_details import AsyncRoutingDetails

        return AsyncRoutingDetails(self)

    @cached_property
    def internal_accounts(self) -> AsyncInternalAccounts:
        from .resources.internal_accounts import AsyncInternalAccounts

        return AsyncInternalAccounts(self)

    @cached_property
    def ledgers(self) -> AsyncLedgers:
        from .resources.ledgers import AsyncLedgers

        return AsyncLedgers(self)

    @cached_property
    def ledger_account_categories(self) -> AsyncLedgerAccountCategories:
        from .resources.ledger_account_categories import AsyncLedgerAccountCategories

        return AsyncLedgerAccountCategories(self)

    @cached_property
    def ledger_accounts(self) -> AsyncLedgerAccounts:
        from .resources.ledger_accounts import AsyncLedgerAccounts

        return AsyncLedgerAccounts(self)

    @cached_property
    def ledger_account_balance_monitors(self) -> AsyncLedgerAccountBalanceMonitors:
        from .resources.ledger_account_balance_monitors import AsyncLedgerAccountBalanceMonitors

        return AsyncLedgerAccountBalanceMonitors(self)

    @cached_property
    def ledger_account_statements(self) -> AsyncLedgerAccountStatements:
        from .resources.ledger_account_statements import AsyncLedgerAccountStatements

        return AsyncLedgerAccountStatements(self)

    @cached_property
    def ledger_entries(self) -> AsyncLedgerEntries:
        from .resources.ledger_entries import AsyncLedgerEntries

        return AsyncLedgerEntries(self)

    @cached_property
    def ledger_transactions(self) -> AsyncLedgerTransactions:
        from .resources.ledger_transactions import AsyncLedgerTransactions

        return AsyncLedgerTransactions(self)

    @cached_property
    def line_items(self) -> AsyncLineItems:
        from .resources.line_items import AsyncLineItems

        return AsyncLineItems(self)

    @cached_property
    def payment_flows(self) -> AsyncPaymentFlows:
        from .resources.payment_flows import AsyncPaymentFlows

        return AsyncPaymentFlows(self)

    @cached_property
    def payment_orders(self) -> AsyncPaymentOrders:
        from .resources.payment_orders import AsyncPaymentOrders

        return AsyncPaymentOrders(self)

    @cached_property
    def payment_references(self) -> AsyncPaymentReferences:
        from .resources.payment_references import AsyncPaymentReferences

        return AsyncPaymentReferences(self)

    @cached_property
    def returns(self) -> AsyncReturns:
        from .resources.returns import AsyncReturns

        return AsyncReturns(self)

    @cached_property
    def transactions(self) -> AsyncTransactions:
        from .resources.transactions import AsyncTransactions

        return AsyncTransactions(self)

    @cached_property
    def validations(self) -> AsyncValidations:
        from .resources.validations import AsyncValidations

        return AsyncValidations(self)

    @cached_property
    def paper_items(self) -> AsyncPaperItems:
        from .resources.paper_items import AsyncPaperItems

        return AsyncPaperItems(self)

    @cached_property
    def virtual_accounts(self) -> AsyncVirtualAccounts:
        from .resources.virtual_accounts import AsyncVirtualAccounts

        return AsyncVirtualAccounts(self)

    @cached_property
    def bulk_requests(self) -> AsyncBulkRequests:
        from .resources.bulk_requests import AsyncBulkRequests

        return AsyncBulkRequests(self)

    @cached_property
    def bulk_results(self) -> AsyncBulkResults:
        from .resources.bulk_results import AsyncBulkResults

        return AsyncBulkResults(self)

    @cached_property
    def ledger_account_settlements(self) -> AsyncLedgerAccountSettlements:
        from .resources.ledger_account_settlements import AsyncLedgerAccountSettlements

        return AsyncLedgerAccountSettlements(self)

    @cached_property
    def foreign_exchange_quotes(self) -> AsyncForeignExchangeQuotes:
        from .resources.foreign_exchange_quotes import AsyncForeignExchangeQuotes

        return AsyncForeignExchangeQuotes(self)

    @cached_property
    def connection_legal_entities(self) -> AsyncConnectionLegalEntities:
        from .resources.connection_legal_entities import AsyncConnectionLegalEntities

        return AsyncConnectionLegalEntities(self)

    @cached_property
    def legal_entities(self) -> AsyncLegalEntities:
        from .resources.legal_entities import AsyncLegalEntities

        return AsyncLegalEntities(self)

    @cached_property
    def legal_entity_associations(self) -> AsyncLegalEntityAssociations:
        from .resources.legal_entity_associations import AsyncLegalEntityAssociations

        return AsyncLegalEntityAssociations(self)

    @cached_property
    def payment_actions(self) -> AsyncPaymentActions:
        from .resources.payment_actions import AsyncPaymentActions

        return AsyncPaymentActions(self)

    @cached_property
    def with_raw_response(self) -> AsyncModernTreasuryWithRawResponse:
        return AsyncModernTreasuryWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModernTreasuryWithStreamedResponse:
        return AsyncModernTreasuryWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="brackets")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        credentials = f"{self.organization_id}:{self.api_key}".encode("ascii")
        header = f"Basic {base64.b64encode(credentials).decode('ascii')}"
        return {"Authorization": header}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        organization_id: str | None = None,
        webhook_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            organization_id=organization_id or self.organization_id,
            webhook_key=webhook_key or self.webhook_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    async def ping(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PingResponse:
        """
        A test endpoint often used to confirm credentials and headers are being passed
        in correctly.
        """
        return await self.get(
            "/api/ping",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PingResponse,
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class ModernTreasuryWithRawResponse:
    _client: ModernTreasury

    def __init__(self, client: ModernTreasury) -> None:
        self._client = client

        self.ping = _legacy_response.to_raw_response_wrapper(
            client.ping,
        )

    @cached_property
    def connections(self) -> connections.ConnectionsWithRawResponse:
        from .resources.connections import ConnectionsWithRawResponse

        return ConnectionsWithRawResponse(self._client.connections)

    @cached_property
    def counterparties(self) -> counterparties.CounterpartiesWithRawResponse:
        from .resources.counterparties import CounterpartiesWithRawResponse

        return CounterpartiesWithRawResponse(self._client.counterparties)

    @cached_property
    def events(self) -> events.EventsWithRawResponse:
        from .resources.events import EventsWithRawResponse

        return EventsWithRawResponse(self._client.events)

    @cached_property
    def expected_payments(self) -> expected_payments.ExpectedPaymentsWithRawResponse:
        from .resources.expected_payments import ExpectedPaymentsWithRawResponse

        return ExpectedPaymentsWithRawResponse(self._client.expected_payments)

    @cached_property
    def external_accounts(self) -> external_accounts.ExternalAccountsWithRawResponse:
        from .resources.external_accounts import ExternalAccountsWithRawResponse

        return ExternalAccountsWithRawResponse(self._client.external_accounts)

    @cached_property
    def incoming_payment_details(self) -> incoming_payment_details.IncomingPaymentDetailsWithRawResponse:
        from .resources.incoming_payment_details import IncomingPaymentDetailsWithRawResponse

        return IncomingPaymentDetailsWithRawResponse(self._client.incoming_payment_details)

    @cached_property
    def invoices(self) -> invoices.InvoicesWithRawResponse:
        from .resources.invoices import InvoicesWithRawResponse

        return InvoicesWithRawResponse(self._client.invoices)

    @cached_property
    def documents(self) -> documents.DocumentsWithRawResponse:
        from .resources.documents import DocumentsWithRawResponse

        return DocumentsWithRawResponse(self._client.documents)

    @cached_property
    def account_collection_flows(self) -> account_collection_flows.AccountCollectionFlowsWithRawResponse:
        from .resources.account_collection_flows import AccountCollectionFlowsWithRawResponse

        return AccountCollectionFlowsWithRawResponse(self._client.account_collection_flows)

    @cached_property
    def account_details(self) -> account_details.AccountDetailsWithRawResponse:
        from .resources.account_details import AccountDetailsWithRawResponse

        return AccountDetailsWithRawResponse(self._client.account_details)

    @cached_property
    def routing_details(self) -> routing_details.RoutingDetailsWithRawResponse:
        from .resources.routing_details import RoutingDetailsWithRawResponse

        return RoutingDetailsWithRawResponse(self._client.routing_details)

    @cached_property
    def internal_accounts(self) -> internal_accounts.InternalAccountsWithRawResponse:
        from .resources.internal_accounts import InternalAccountsWithRawResponse

        return InternalAccountsWithRawResponse(self._client.internal_accounts)

    @cached_property
    def ledgers(self) -> ledgers.LedgersWithRawResponse:
        from .resources.ledgers import LedgersWithRawResponse

        return LedgersWithRawResponse(self._client.ledgers)

    @cached_property
    def ledger_account_categories(self) -> ledger_account_categories.LedgerAccountCategoriesWithRawResponse:
        from .resources.ledger_account_categories import LedgerAccountCategoriesWithRawResponse

        return LedgerAccountCategoriesWithRawResponse(self._client.ledger_account_categories)

    @cached_property
    def ledger_accounts(self) -> ledger_accounts.LedgerAccountsWithRawResponse:
        from .resources.ledger_accounts import LedgerAccountsWithRawResponse

        return LedgerAccountsWithRawResponse(self._client.ledger_accounts)

    @cached_property
    def ledger_account_balance_monitors(
        self,
    ) -> ledger_account_balance_monitors.LedgerAccountBalanceMonitorsWithRawResponse:
        from .resources.ledger_account_balance_monitors import LedgerAccountBalanceMonitorsWithRawResponse

        return LedgerAccountBalanceMonitorsWithRawResponse(self._client.ledger_account_balance_monitors)

    @cached_property
    def ledger_account_statements(self) -> ledger_account_statements.LedgerAccountStatementsWithRawResponse:
        from .resources.ledger_account_statements import LedgerAccountStatementsWithRawResponse

        return LedgerAccountStatementsWithRawResponse(self._client.ledger_account_statements)

    @cached_property
    def ledger_entries(self) -> ledger_entries.LedgerEntriesWithRawResponse:
        from .resources.ledger_entries import LedgerEntriesWithRawResponse

        return LedgerEntriesWithRawResponse(self._client.ledger_entries)

    @cached_property
    def ledger_transactions(self) -> ledger_transactions.LedgerTransactionsWithRawResponse:
        from .resources.ledger_transactions import LedgerTransactionsWithRawResponse

        return LedgerTransactionsWithRawResponse(self._client.ledger_transactions)

    @cached_property
    def line_items(self) -> line_items.LineItemsWithRawResponse:
        from .resources.line_items import LineItemsWithRawResponse

        return LineItemsWithRawResponse(self._client.line_items)

    @cached_property
    def payment_flows(self) -> payment_flows.PaymentFlowsWithRawResponse:
        from .resources.payment_flows import PaymentFlowsWithRawResponse

        return PaymentFlowsWithRawResponse(self._client.payment_flows)

    @cached_property
    def payment_orders(self) -> payment_orders.PaymentOrdersWithRawResponse:
        from .resources.payment_orders import PaymentOrdersWithRawResponse

        return PaymentOrdersWithRawResponse(self._client.payment_orders)

    @cached_property
    def payment_references(self) -> payment_references.PaymentReferencesWithRawResponse:
        from .resources.payment_references import PaymentReferencesWithRawResponse

        return PaymentReferencesWithRawResponse(self._client.payment_references)

    @cached_property
    def returns(self) -> returns.ReturnsWithRawResponse:
        from .resources.returns import ReturnsWithRawResponse

        return ReturnsWithRawResponse(self._client.returns)

    @cached_property
    def transactions(self) -> transactions.TransactionsWithRawResponse:
        from .resources.transactions import TransactionsWithRawResponse

        return TransactionsWithRawResponse(self._client.transactions)

    @cached_property
    def validations(self) -> validations.ValidationsWithRawResponse:
        from .resources.validations import ValidationsWithRawResponse

        return ValidationsWithRawResponse(self._client.validations)

    @cached_property
    def paper_items(self) -> paper_items.PaperItemsWithRawResponse:
        from .resources.paper_items import PaperItemsWithRawResponse

        return PaperItemsWithRawResponse(self._client.paper_items)

    @cached_property
    def virtual_accounts(self) -> virtual_accounts.VirtualAccountsWithRawResponse:
        from .resources.virtual_accounts import VirtualAccountsWithRawResponse

        return VirtualAccountsWithRawResponse(self._client.virtual_accounts)

    @cached_property
    def bulk_requests(self) -> bulk_requests.BulkRequestsWithRawResponse:
        from .resources.bulk_requests import BulkRequestsWithRawResponse

        return BulkRequestsWithRawResponse(self._client.bulk_requests)

    @cached_property
    def bulk_results(self) -> bulk_results.BulkResultsWithRawResponse:
        from .resources.bulk_results import BulkResultsWithRawResponse

        return BulkResultsWithRawResponse(self._client.bulk_results)

    @cached_property
    def ledger_account_settlements(self) -> ledger_account_settlements.LedgerAccountSettlementsWithRawResponse:
        from .resources.ledger_account_settlements import LedgerAccountSettlementsWithRawResponse

        return LedgerAccountSettlementsWithRawResponse(self._client.ledger_account_settlements)

    @cached_property
    def foreign_exchange_quotes(self) -> foreign_exchange_quotes.ForeignExchangeQuotesWithRawResponse:
        from .resources.foreign_exchange_quotes import ForeignExchangeQuotesWithRawResponse

        return ForeignExchangeQuotesWithRawResponse(self._client.foreign_exchange_quotes)

    @cached_property
    def connection_legal_entities(self) -> connection_legal_entities.ConnectionLegalEntitiesWithRawResponse:
        from .resources.connection_legal_entities import ConnectionLegalEntitiesWithRawResponse

        return ConnectionLegalEntitiesWithRawResponse(self._client.connection_legal_entities)

    @cached_property
    def legal_entities(self) -> legal_entities.LegalEntitiesWithRawResponse:
        from .resources.legal_entities import LegalEntitiesWithRawResponse

        return LegalEntitiesWithRawResponse(self._client.legal_entities)

    @cached_property
    def legal_entity_associations(self) -> legal_entity_associations.LegalEntityAssociationsWithRawResponse:
        from .resources.legal_entity_associations import LegalEntityAssociationsWithRawResponse

        return LegalEntityAssociationsWithRawResponse(self._client.legal_entity_associations)

    @cached_property
    def payment_actions(self) -> payment_actions.PaymentActionsWithRawResponse:
        from .resources.payment_actions import PaymentActionsWithRawResponse

        return PaymentActionsWithRawResponse(self._client.payment_actions)


class AsyncModernTreasuryWithRawResponse:
    _client: AsyncModernTreasury

    def __init__(self, client: AsyncModernTreasury) -> None:
        self._client = client

        self.ping = _legacy_response.async_to_raw_response_wrapper(
            client.ping,
        )

    @cached_property
    def connections(self) -> connections.AsyncConnectionsWithRawResponse:
        from .resources.connections import AsyncConnectionsWithRawResponse

        return AsyncConnectionsWithRawResponse(self._client.connections)

    @cached_property
    def counterparties(self) -> counterparties.AsyncCounterpartiesWithRawResponse:
        from .resources.counterparties import AsyncCounterpartiesWithRawResponse

        return AsyncCounterpartiesWithRawResponse(self._client.counterparties)

    @cached_property
    def events(self) -> events.AsyncEventsWithRawResponse:
        from .resources.events import AsyncEventsWithRawResponse

        return AsyncEventsWithRawResponse(self._client.events)

    @cached_property
    def expected_payments(self) -> expected_payments.AsyncExpectedPaymentsWithRawResponse:
        from .resources.expected_payments import AsyncExpectedPaymentsWithRawResponse

        return AsyncExpectedPaymentsWithRawResponse(self._client.expected_payments)

    @cached_property
    def external_accounts(self) -> external_accounts.AsyncExternalAccountsWithRawResponse:
        from .resources.external_accounts import AsyncExternalAccountsWithRawResponse

        return AsyncExternalAccountsWithRawResponse(self._client.external_accounts)

    @cached_property
    def incoming_payment_details(self) -> incoming_payment_details.AsyncIncomingPaymentDetailsWithRawResponse:
        from .resources.incoming_payment_details import AsyncIncomingPaymentDetailsWithRawResponse

        return AsyncIncomingPaymentDetailsWithRawResponse(self._client.incoming_payment_details)

    @cached_property
    def invoices(self) -> invoices.AsyncInvoicesWithRawResponse:
        from .resources.invoices import AsyncInvoicesWithRawResponse

        return AsyncInvoicesWithRawResponse(self._client.invoices)

    @cached_property
    def documents(self) -> documents.AsyncDocumentsWithRawResponse:
        from .resources.documents import AsyncDocumentsWithRawResponse

        return AsyncDocumentsWithRawResponse(self._client.documents)

    @cached_property
    def account_collection_flows(self) -> account_collection_flows.AsyncAccountCollectionFlowsWithRawResponse:
        from .resources.account_collection_flows import AsyncAccountCollectionFlowsWithRawResponse

        return AsyncAccountCollectionFlowsWithRawResponse(self._client.account_collection_flows)

    @cached_property
    def account_details(self) -> account_details.AsyncAccountDetailsWithRawResponse:
        from .resources.account_details import AsyncAccountDetailsWithRawResponse

        return AsyncAccountDetailsWithRawResponse(self._client.account_details)

    @cached_property
    def routing_details(self) -> routing_details.AsyncRoutingDetailsWithRawResponse:
        from .resources.routing_details import AsyncRoutingDetailsWithRawResponse

        return AsyncRoutingDetailsWithRawResponse(self._client.routing_details)

    @cached_property
    def internal_accounts(self) -> internal_accounts.AsyncInternalAccountsWithRawResponse:
        from .resources.internal_accounts import AsyncInternalAccountsWithRawResponse

        return AsyncInternalAccountsWithRawResponse(self._client.internal_accounts)

    @cached_property
    def ledgers(self) -> ledgers.AsyncLedgersWithRawResponse:
        from .resources.ledgers import AsyncLedgersWithRawResponse

        return AsyncLedgersWithRawResponse(self._client.ledgers)

    @cached_property
    def ledger_account_categories(self) -> ledger_account_categories.AsyncLedgerAccountCategoriesWithRawResponse:
        from .resources.ledger_account_categories import AsyncLedgerAccountCategoriesWithRawResponse

        return AsyncLedgerAccountCategoriesWithRawResponse(self._client.ledger_account_categories)

    @cached_property
    def ledger_accounts(self) -> ledger_accounts.AsyncLedgerAccountsWithRawResponse:
        from .resources.ledger_accounts import AsyncLedgerAccountsWithRawResponse

        return AsyncLedgerAccountsWithRawResponse(self._client.ledger_accounts)

    @cached_property
    def ledger_account_balance_monitors(
        self,
    ) -> ledger_account_balance_monitors.AsyncLedgerAccountBalanceMonitorsWithRawResponse:
        from .resources.ledger_account_balance_monitors import AsyncLedgerAccountBalanceMonitorsWithRawResponse

        return AsyncLedgerAccountBalanceMonitorsWithRawResponse(self._client.ledger_account_balance_monitors)

    @cached_property
    def ledger_account_statements(self) -> ledger_account_statements.AsyncLedgerAccountStatementsWithRawResponse:
        from .resources.ledger_account_statements import AsyncLedgerAccountStatementsWithRawResponse

        return AsyncLedgerAccountStatementsWithRawResponse(self._client.ledger_account_statements)

    @cached_property
    def ledger_entries(self) -> ledger_entries.AsyncLedgerEntriesWithRawResponse:
        from .resources.ledger_entries import AsyncLedgerEntriesWithRawResponse

        return AsyncLedgerEntriesWithRawResponse(self._client.ledger_entries)

    @cached_property
    def ledger_transactions(self) -> ledger_transactions.AsyncLedgerTransactionsWithRawResponse:
        from .resources.ledger_transactions import AsyncLedgerTransactionsWithRawResponse

        return AsyncLedgerTransactionsWithRawResponse(self._client.ledger_transactions)

    @cached_property
    def line_items(self) -> line_items.AsyncLineItemsWithRawResponse:
        from .resources.line_items import AsyncLineItemsWithRawResponse

        return AsyncLineItemsWithRawResponse(self._client.line_items)

    @cached_property
    def payment_flows(self) -> payment_flows.AsyncPaymentFlowsWithRawResponse:
        from .resources.payment_flows import AsyncPaymentFlowsWithRawResponse

        return AsyncPaymentFlowsWithRawResponse(self._client.payment_flows)

    @cached_property
    def payment_orders(self) -> payment_orders.AsyncPaymentOrdersWithRawResponse:
        from .resources.payment_orders import AsyncPaymentOrdersWithRawResponse

        return AsyncPaymentOrdersWithRawResponse(self._client.payment_orders)

    @cached_property
    def payment_references(self) -> payment_references.AsyncPaymentReferencesWithRawResponse:
        from .resources.payment_references import AsyncPaymentReferencesWithRawResponse

        return AsyncPaymentReferencesWithRawResponse(self._client.payment_references)

    @cached_property
    def returns(self) -> returns.AsyncReturnsWithRawResponse:
        from .resources.returns import AsyncReturnsWithRawResponse

        return AsyncReturnsWithRawResponse(self._client.returns)

    @cached_property
    def transactions(self) -> transactions.AsyncTransactionsWithRawResponse:
        from .resources.transactions import AsyncTransactionsWithRawResponse

        return AsyncTransactionsWithRawResponse(self._client.transactions)

    @cached_property
    def validations(self) -> validations.AsyncValidationsWithRawResponse:
        from .resources.validations import AsyncValidationsWithRawResponse

        return AsyncValidationsWithRawResponse(self._client.validations)

    @cached_property
    def paper_items(self) -> paper_items.AsyncPaperItemsWithRawResponse:
        from .resources.paper_items import AsyncPaperItemsWithRawResponse

        return AsyncPaperItemsWithRawResponse(self._client.paper_items)

    @cached_property
    def virtual_accounts(self) -> virtual_accounts.AsyncVirtualAccountsWithRawResponse:
        from .resources.virtual_accounts import AsyncVirtualAccountsWithRawResponse

        return AsyncVirtualAccountsWithRawResponse(self._client.virtual_accounts)

    @cached_property
    def bulk_requests(self) -> bulk_requests.AsyncBulkRequestsWithRawResponse:
        from .resources.bulk_requests import AsyncBulkRequestsWithRawResponse

        return AsyncBulkRequestsWithRawResponse(self._client.bulk_requests)

    @cached_property
    def bulk_results(self) -> bulk_results.AsyncBulkResultsWithRawResponse:
        from .resources.bulk_results import AsyncBulkResultsWithRawResponse

        return AsyncBulkResultsWithRawResponse(self._client.bulk_results)

    @cached_property
    def ledger_account_settlements(self) -> ledger_account_settlements.AsyncLedgerAccountSettlementsWithRawResponse:
        from .resources.ledger_account_settlements import AsyncLedgerAccountSettlementsWithRawResponse

        return AsyncLedgerAccountSettlementsWithRawResponse(self._client.ledger_account_settlements)

    @cached_property
    def foreign_exchange_quotes(self) -> foreign_exchange_quotes.AsyncForeignExchangeQuotesWithRawResponse:
        from .resources.foreign_exchange_quotes import AsyncForeignExchangeQuotesWithRawResponse

        return AsyncForeignExchangeQuotesWithRawResponse(self._client.foreign_exchange_quotes)

    @cached_property
    def connection_legal_entities(self) -> connection_legal_entities.AsyncConnectionLegalEntitiesWithRawResponse:
        from .resources.connection_legal_entities import AsyncConnectionLegalEntitiesWithRawResponse

        return AsyncConnectionLegalEntitiesWithRawResponse(self._client.connection_legal_entities)

    @cached_property
    def legal_entities(self) -> legal_entities.AsyncLegalEntitiesWithRawResponse:
        from .resources.legal_entities import AsyncLegalEntitiesWithRawResponse

        return AsyncLegalEntitiesWithRawResponse(self._client.legal_entities)

    @cached_property
    def legal_entity_associations(self) -> legal_entity_associations.AsyncLegalEntityAssociationsWithRawResponse:
        from .resources.legal_entity_associations import AsyncLegalEntityAssociationsWithRawResponse

        return AsyncLegalEntityAssociationsWithRawResponse(self._client.legal_entity_associations)

    @cached_property
    def payment_actions(self) -> payment_actions.AsyncPaymentActionsWithRawResponse:
        from .resources.payment_actions import AsyncPaymentActionsWithRawResponse

        return AsyncPaymentActionsWithRawResponse(self._client.payment_actions)


class ModernTreasuryWithStreamedResponse:
    _client: ModernTreasury

    def __init__(self, client: ModernTreasury) -> None:
        self._client = client

        self.ping = to_streamed_response_wrapper(
            client.ping,
        )

    @cached_property
    def connections(self) -> connections.ConnectionsWithStreamingResponse:
        from .resources.connections import ConnectionsWithStreamingResponse

        return ConnectionsWithStreamingResponse(self._client.connections)

    @cached_property
    def counterparties(self) -> counterparties.CounterpartiesWithStreamingResponse:
        from .resources.counterparties import CounterpartiesWithStreamingResponse

        return CounterpartiesWithStreamingResponse(self._client.counterparties)

    @cached_property
    def events(self) -> events.EventsWithStreamingResponse:
        from .resources.events import EventsWithStreamingResponse

        return EventsWithStreamingResponse(self._client.events)

    @cached_property
    def expected_payments(self) -> expected_payments.ExpectedPaymentsWithStreamingResponse:
        from .resources.expected_payments import ExpectedPaymentsWithStreamingResponse

        return ExpectedPaymentsWithStreamingResponse(self._client.expected_payments)

    @cached_property
    def external_accounts(self) -> external_accounts.ExternalAccountsWithStreamingResponse:
        from .resources.external_accounts import ExternalAccountsWithStreamingResponse

        return ExternalAccountsWithStreamingResponse(self._client.external_accounts)

    @cached_property
    def incoming_payment_details(self) -> incoming_payment_details.IncomingPaymentDetailsWithStreamingResponse:
        from .resources.incoming_payment_details import IncomingPaymentDetailsWithStreamingResponse

        return IncomingPaymentDetailsWithStreamingResponse(self._client.incoming_payment_details)

    @cached_property
    def invoices(self) -> invoices.InvoicesWithStreamingResponse:
        from .resources.invoices import InvoicesWithStreamingResponse

        return InvoicesWithStreamingResponse(self._client.invoices)

    @cached_property
    def documents(self) -> documents.DocumentsWithStreamingResponse:
        from .resources.documents import DocumentsWithStreamingResponse

        return DocumentsWithStreamingResponse(self._client.documents)

    @cached_property
    def account_collection_flows(self) -> account_collection_flows.AccountCollectionFlowsWithStreamingResponse:
        from .resources.account_collection_flows import AccountCollectionFlowsWithStreamingResponse

        return AccountCollectionFlowsWithStreamingResponse(self._client.account_collection_flows)

    @cached_property
    def account_details(self) -> account_details.AccountDetailsWithStreamingResponse:
        from .resources.account_details import AccountDetailsWithStreamingResponse

        return AccountDetailsWithStreamingResponse(self._client.account_details)

    @cached_property
    def routing_details(self) -> routing_details.RoutingDetailsWithStreamingResponse:
        from .resources.routing_details import RoutingDetailsWithStreamingResponse

        return RoutingDetailsWithStreamingResponse(self._client.routing_details)

    @cached_property
    def internal_accounts(self) -> internal_accounts.InternalAccountsWithStreamingResponse:
        from .resources.internal_accounts import InternalAccountsWithStreamingResponse

        return InternalAccountsWithStreamingResponse(self._client.internal_accounts)

    @cached_property
    def ledgers(self) -> ledgers.LedgersWithStreamingResponse:
        from .resources.ledgers import LedgersWithStreamingResponse

        return LedgersWithStreamingResponse(self._client.ledgers)

    @cached_property
    def ledger_account_categories(self) -> ledger_account_categories.LedgerAccountCategoriesWithStreamingResponse:
        from .resources.ledger_account_categories import LedgerAccountCategoriesWithStreamingResponse

        return LedgerAccountCategoriesWithStreamingResponse(self._client.ledger_account_categories)

    @cached_property
    def ledger_accounts(self) -> ledger_accounts.LedgerAccountsWithStreamingResponse:
        from .resources.ledger_accounts import LedgerAccountsWithStreamingResponse

        return LedgerAccountsWithStreamingResponse(self._client.ledger_accounts)

    @cached_property
    def ledger_account_balance_monitors(
        self,
    ) -> ledger_account_balance_monitors.LedgerAccountBalanceMonitorsWithStreamingResponse:
        from .resources.ledger_account_balance_monitors import LedgerAccountBalanceMonitorsWithStreamingResponse

        return LedgerAccountBalanceMonitorsWithStreamingResponse(self._client.ledger_account_balance_monitors)

    @cached_property
    def ledger_account_statements(self) -> ledger_account_statements.LedgerAccountStatementsWithStreamingResponse:
        from .resources.ledger_account_statements import LedgerAccountStatementsWithStreamingResponse

        return LedgerAccountStatementsWithStreamingResponse(self._client.ledger_account_statements)

    @cached_property
    def ledger_entries(self) -> ledger_entries.LedgerEntriesWithStreamingResponse:
        from .resources.ledger_entries import LedgerEntriesWithStreamingResponse

        return LedgerEntriesWithStreamingResponse(self._client.ledger_entries)

    @cached_property
    def ledger_transactions(self) -> ledger_transactions.LedgerTransactionsWithStreamingResponse:
        from .resources.ledger_transactions import LedgerTransactionsWithStreamingResponse

        return LedgerTransactionsWithStreamingResponse(self._client.ledger_transactions)

    @cached_property
    def line_items(self) -> line_items.LineItemsWithStreamingResponse:
        from .resources.line_items import LineItemsWithStreamingResponse

        return LineItemsWithStreamingResponse(self._client.line_items)

    @cached_property
    def payment_flows(self) -> payment_flows.PaymentFlowsWithStreamingResponse:
        from .resources.payment_flows import PaymentFlowsWithStreamingResponse

        return PaymentFlowsWithStreamingResponse(self._client.payment_flows)

    @cached_property
    def payment_orders(self) -> payment_orders.PaymentOrdersWithStreamingResponse:
        from .resources.payment_orders import PaymentOrdersWithStreamingResponse

        return PaymentOrdersWithStreamingResponse(self._client.payment_orders)

    @cached_property
    def payment_references(self) -> payment_references.PaymentReferencesWithStreamingResponse:
        from .resources.payment_references import PaymentReferencesWithStreamingResponse

        return PaymentReferencesWithStreamingResponse(self._client.payment_references)

    @cached_property
    def returns(self) -> returns.ReturnsWithStreamingResponse:
        from .resources.returns import ReturnsWithStreamingResponse

        return ReturnsWithStreamingResponse(self._client.returns)

    @cached_property
    def transactions(self) -> transactions.TransactionsWithStreamingResponse:
        from .resources.transactions import TransactionsWithStreamingResponse

        return TransactionsWithStreamingResponse(self._client.transactions)

    @cached_property
    def validations(self) -> validations.ValidationsWithStreamingResponse:
        from .resources.validations import ValidationsWithStreamingResponse

        return ValidationsWithStreamingResponse(self._client.validations)

    @cached_property
    def paper_items(self) -> paper_items.PaperItemsWithStreamingResponse:
        from .resources.paper_items import PaperItemsWithStreamingResponse

        return PaperItemsWithStreamingResponse(self._client.paper_items)

    @cached_property
    def virtual_accounts(self) -> virtual_accounts.VirtualAccountsWithStreamingResponse:
        from .resources.virtual_accounts import VirtualAccountsWithStreamingResponse

        return VirtualAccountsWithStreamingResponse(self._client.virtual_accounts)

    @cached_property
    def bulk_requests(self) -> bulk_requests.BulkRequestsWithStreamingResponse:
        from .resources.bulk_requests import BulkRequestsWithStreamingResponse

        return BulkRequestsWithStreamingResponse(self._client.bulk_requests)

    @cached_property
    def bulk_results(self) -> bulk_results.BulkResultsWithStreamingResponse:
        from .resources.bulk_results import BulkResultsWithStreamingResponse

        return BulkResultsWithStreamingResponse(self._client.bulk_results)

    @cached_property
    def ledger_account_settlements(self) -> ledger_account_settlements.LedgerAccountSettlementsWithStreamingResponse:
        from .resources.ledger_account_settlements import LedgerAccountSettlementsWithStreamingResponse

        return LedgerAccountSettlementsWithStreamingResponse(self._client.ledger_account_settlements)

    @cached_property
    def foreign_exchange_quotes(self) -> foreign_exchange_quotes.ForeignExchangeQuotesWithStreamingResponse:
        from .resources.foreign_exchange_quotes import ForeignExchangeQuotesWithStreamingResponse

        return ForeignExchangeQuotesWithStreamingResponse(self._client.foreign_exchange_quotes)

    @cached_property
    def connection_legal_entities(self) -> connection_legal_entities.ConnectionLegalEntitiesWithStreamingResponse:
        from .resources.connection_legal_entities import ConnectionLegalEntitiesWithStreamingResponse

        return ConnectionLegalEntitiesWithStreamingResponse(self._client.connection_legal_entities)

    @cached_property
    def legal_entities(self) -> legal_entities.LegalEntitiesWithStreamingResponse:
        from .resources.legal_entities import LegalEntitiesWithStreamingResponse

        return LegalEntitiesWithStreamingResponse(self._client.legal_entities)

    @cached_property
    def legal_entity_associations(self) -> legal_entity_associations.LegalEntityAssociationsWithStreamingResponse:
        from .resources.legal_entity_associations import LegalEntityAssociationsWithStreamingResponse

        return LegalEntityAssociationsWithStreamingResponse(self._client.legal_entity_associations)

    @cached_property
    def payment_actions(self) -> payment_actions.PaymentActionsWithStreamingResponse:
        from .resources.payment_actions import PaymentActionsWithStreamingResponse

        return PaymentActionsWithStreamingResponse(self._client.payment_actions)


class AsyncModernTreasuryWithStreamedResponse:
    _client: AsyncModernTreasury

    def __init__(self, client: AsyncModernTreasury) -> None:
        self._client = client

        self.ping = async_to_streamed_response_wrapper(
            client.ping,
        )

    @cached_property
    def connections(self) -> connections.AsyncConnectionsWithStreamingResponse:
        from .resources.connections import AsyncConnectionsWithStreamingResponse

        return AsyncConnectionsWithStreamingResponse(self._client.connections)

    @cached_property
    def counterparties(self) -> counterparties.AsyncCounterpartiesWithStreamingResponse:
        from .resources.counterparties import AsyncCounterpartiesWithStreamingResponse

        return AsyncCounterpartiesWithStreamingResponse(self._client.counterparties)

    @cached_property
    def events(self) -> events.AsyncEventsWithStreamingResponse:
        from .resources.events import AsyncEventsWithStreamingResponse

        return AsyncEventsWithStreamingResponse(self._client.events)

    @cached_property
    def expected_payments(self) -> expected_payments.AsyncExpectedPaymentsWithStreamingResponse:
        from .resources.expected_payments import AsyncExpectedPaymentsWithStreamingResponse

        return AsyncExpectedPaymentsWithStreamingResponse(self._client.expected_payments)

    @cached_property
    def external_accounts(self) -> external_accounts.AsyncExternalAccountsWithStreamingResponse:
        from .resources.external_accounts import AsyncExternalAccountsWithStreamingResponse

        return AsyncExternalAccountsWithStreamingResponse(self._client.external_accounts)

    @cached_property
    def incoming_payment_details(self) -> incoming_payment_details.AsyncIncomingPaymentDetailsWithStreamingResponse:
        from .resources.incoming_payment_details import AsyncIncomingPaymentDetailsWithStreamingResponse

        return AsyncIncomingPaymentDetailsWithStreamingResponse(self._client.incoming_payment_details)

    @cached_property
    def invoices(self) -> invoices.AsyncInvoicesWithStreamingResponse:
        from .resources.invoices import AsyncInvoicesWithStreamingResponse

        return AsyncInvoicesWithStreamingResponse(self._client.invoices)

    @cached_property
    def documents(self) -> documents.AsyncDocumentsWithStreamingResponse:
        from .resources.documents import AsyncDocumentsWithStreamingResponse

        return AsyncDocumentsWithStreamingResponse(self._client.documents)

    @cached_property
    def account_collection_flows(self) -> account_collection_flows.AsyncAccountCollectionFlowsWithStreamingResponse:
        from .resources.account_collection_flows import AsyncAccountCollectionFlowsWithStreamingResponse

        return AsyncAccountCollectionFlowsWithStreamingResponse(self._client.account_collection_flows)

    @cached_property
    def account_details(self) -> account_details.AsyncAccountDetailsWithStreamingResponse:
        from .resources.account_details import AsyncAccountDetailsWithStreamingResponse

        return AsyncAccountDetailsWithStreamingResponse(self._client.account_details)

    @cached_property
    def routing_details(self) -> routing_details.AsyncRoutingDetailsWithStreamingResponse:
        from .resources.routing_details import AsyncRoutingDetailsWithStreamingResponse

        return AsyncRoutingDetailsWithStreamingResponse(self._client.routing_details)

    @cached_property
    def internal_accounts(self) -> internal_accounts.AsyncInternalAccountsWithStreamingResponse:
        from .resources.internal_accounts import AsyncInternalAccountsWithStreamingResponse

        return AsyncInternalAccountsWithStreamingResponse(self._client.internal_accounts)

    @cached_property
    def ledgers(self) -> ledgers.AsyncLedgersWithStreamingResponse:
        from .resources.ledgers import AsyncLedgersWithStreamingResponse

        return AsyncLedgersWithStreamingResponse(self._client.ledgers)

    @cached_property
    def ledger_account_categories(self) -> ledger_account_categories.AsyncLedgerAccountCategoriesWithStreamingResponse:
        from .resources.ledger_account_categories import AsyncLedgerAccountCategoriesWithStreamingResponse

        return AsyncLedgerAccountCategoriesWithStreamingResponse(self._client.ledger_account_categories)

    @cached_property
    def ledger_accounts(self) -> ledger_accounts.AsyncLedgerAccountsWithStreamingResponse:
        from .resources.ledger_accounts import AsyncLedgerAccountsWithStreamingResponse

        return AsyncLedgerAccountsWithStreamingResponse(self._client.ledger_accounts)

    @cached_property
    def ledger_account_balance_monitors(
        self,
    ) -> ledger_account_balance_monitors.AsyncLedgerAccountBalanceMonitorsWithStreamingResponse:
        from .resources.ledger_account_balance_monitors import AsyncLedgerAccountBalanceMonitorsWithStreamingResponse

        return AsyncLedgerAccountBalanceMonitorsWithStreamingResponse(self._client.ledger_account_balance_monitors)

    @cached_property
    def ledger_account_statements(self) -> ledger_account_statements.AsyncLedgerAccountStatementsWithStreamingResponse:
        from .resources.ledger_account_statements import AsyncLedgerAccountStatementsWithStreamingResponse

        return AsyncLedgerAccountStatementsWithStreamingResponse(self._client.ledger_account_statements)

    @cached_property
    def ledger_entries(self) -> ledger_entries.AsyncLedgerEntriesWithStreamingResponse:
        from .resources.ledger_entries import AsyncLedgerEntriesWithStreamingResponse

        return AsyncLedgerEntriesWithStreamingResponse(self._client.ledger_entries)

    @cached_property
    def ledger_transactions(self) -> ledger_transactions.AsyncLedgerTransactionsWithStreamingResponse:
        from .resources.ledger_transactions import AsyncLedgerTransactionsWithStreamingResponse

        return AsyncLedgerTransactionsWithStreamingResponse(self._client.ledger_transactions)

    @cached_property
    def line_items(self) -> line_items.AsyncLineItemsWithStreamingResponse:
        from .resources.line_items import AsyncLineItemsWithStreamingResponse

        return AsyncLineItemsWithStreamingResponse(self._client.line_items)

    @cached_property
    def payment_flows(self) -> payment_flows.AsyncPaymentFlowsWithStreamingResponse:
        from .resources.payment_flows import AsyncPaymentFlowsWithStreamingResponse

        return AsyncPaymentFlowsWithStreamingResponse(self._client.payment_flows)

    @cached_property
    def payment_orders(self) -> payment_orders.AsyncPaymentOrdersWithStreamingResponse:
        from .resources.payment_orders import AsyncPaymentOrdersWithStreamingResponse

        return AsyncPaymentOrdersWithStreamingResponse(self._client.payment_orders)

    @cached_property
    def payment_references(self) -> payment_references.AsyncPaymentReferencesWithStreamingResponse:
        from .resources.payment_references import AsyncPaymentReferencesWithStreamingResponse

        return AsyncPaymentReferencesWithStreamingResponse(self._client.payment_references)

    @cached_property
    def returns(self) -> returns.AsyncReturnsWithStreamingResponse:
        from .resources.returns import AsyncReturnsWithStreamingResponse

        return AsyncReturnsWithStreamingResponse(self._client.returns)

    @cached_property
    def transactions(self) -> transactions.AsyncTransactionsWithStreamingResponse:
        from .resources.transactions import AsyncTransactionsWithStreamingResponse

        return AsyncTransactionsWithStreamingResponse(self._client.transactions)

    @cached_property
    def validations(self) -> validations.AsyncValidationsWithStreamingResponse:
        from .resources.validations import AsyncValidationsWithStreamingResponse

        return AsyncValidationsWithStreamingResponse(self._client.validations)

    @cached_property
    def paper_items(self) -> paper_items.AsyncPaperItemsWithStreamingResponse:
        from .resources.paper_items import AsyncPaperItemsWithStreamingResponse

        return AsyncPaperItemsWithStreamingResponse(self._client.paper_items)

    @cached_property
    def virtual_accounts(self) -> virtual_accounts.AsyncVirtualAccountsWithStreamingResponse:
        from .resources.virtual_accounts import AsyncVirtualAccountsWithStreamingResponse

        return AsyncVirtualAccountsWithStreamingResponse(self._client.virtual_accounts)

    @cached_property
    def bulk_requests(self) -> bulk_requests.AsyncBulkRequestsWithStreamingResponse:
        from .resources.bulk_requests import AsyncBulkRequestsWithStreamingResponse

        return AsyncBulkRequestsWithStreamingResponse(self._client.bulk_requests)

    @cached_property
    def bulk_results(self) -> bulk_results.AsyncBulkResultsWithStreamingResponse:
        from .resources.bulk_results import AsyncBulkResultsWithStreamingResponse

        return AsyncBulkResultsWithStreamingResponse(self._client.bulk_results)

    @cached_property
    def ledger_account_settlements(
        self,
    ) -> ledger_account_settlements.AsyncLedgerAccountSettlementsWithStreamingResponse:
        from .resources.ledger_account_settlements import AsyncLedgerAccountSettlementsWithStreamingResponse

        return AsyncLedgerAccountSettlementsWithStreamingResponse(self._client.ledger_account_settlements)

    @cached_property
    def foreign_exchange_quotes(self) -> foreign_exchange_quotes.AsyncForeignExchangeQuotesWithStreamingResponse:
        from .resources.foreign_exchange_quotes import AsyncForeignExchangeQuotesWithStreamingResponse

        return AsyncForeignExchangeQuotesWithStreamingResponse(self._client.foreign_exchange_quotes)

    @cached_property
    def connection_legal_entities(self) -> connection_legal_entities.AsyncConnectionLegalEntitiesWithStreamingResponse:
        from .resources.connection_legal_entities import AsyncConnectionLegalEntitiesWithStreamingResponse

        return AsyncConnectionLegalEntitiesWithStreamingResponse(self._client.connection_legal_entities)

    @cached_property
    def legal_entities(self) -> legal_entities.AsyncLegalEntitiesWithStreamingResponse:
        from .resources.legal_entities import AsyncLegalEntitiesWithStreamingResponse

        return AsyncLegalEntitiesWithStreamingResponse(self._client.legal_entities)

    @cached_property
    def legal_entity_associations(self) -> legal_entity_associations.AsyncLegalEntityAssociationsWithStreamingResponse:
        from .resources.legal_entity_associations import AsyncLegalEntityAssociationsWithStreamingResponse

        return AsyncLegalEntityAssociationsWithStreamingResponse(self._client.legal_entity_associations)

    @cached_property
    def payment_actions(self) -> payment_actions.AsyncPaymentActionsWithStreamingResponse:
        from .resources.payment_actions import AsyncPaymentActionsWithStreamingResponse

        return AsyncPaymentActionsWithStreamingResponse(self._client.payment_actions)


Client = ModernTreasury

AsyncClient = AsyncModernTreasury



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .currency import Currency as Currency
from .accounts_type import AccountsType as AccountsType
from .async_response import AsyncResponse as AsyncResponse
from .transaction_direction import TransactionDirection as TransactionDirection
from .legal_entity_compliance_detail import LegalEntityComplianceDetail as LegalEntityComplianceDetail
from .legal_entity_industry_classification import LegalEntityIndustryClassification as LegalEntityIndustryClassification



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import TypedDict

__all__ = ["VirtualAccountUpdateParams"]


class VirtualAccountUpdateParams(TypedDict, total=False):
    counterparty_id: str

    ledger_account_id: str
    """The ledger account that you'd like to link to the virtual account."""

    metadata: Dict[str, str]

    name: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .invoice_line_item import InvoiceLineItem as InvoiceLineItem
from .line_item_list_params import LineItemListParams as LineItemListParams
from .line_item_create_params import LineItemCreateParams as LineItemCreateParams
from .line_item_update_params import LineItemUpdateParams as LineItemUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import date, datetime
from typing_extensions import Literal

from .._models import BaseModel
from .bank_settings import BankSettings
from .legal_entity_association import LegalEntityAssociation
from .wealth_and_employment_details import WealthAndEmploymentDetails
from .shared.legal_entity_compliance_detail import LegalEntityComplianceDetail
from .shared.legal_entity_industry_classification import LegalEntityIndustryClassification

__all__ = ["LegalEntity", "Addresses", "Address", "Identifications", "Identification", "PhoneNumbers", "PhoneNumber"]


class Address(BaseModel):
    id: str

    address_types: List[Literal["business", "mailing", "other", "po_box", "residential"]]
    """The types of this address."""

    country: Optional[str] = None
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    created_at: datetime

    discarded_at: Optional[datetime] = None

    line1: Optional[str] = None

    line2: Optional[str] = None

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    locality: Optional[str] = None
    """Locality or City."""

    object: str

    postal_code: Optional[str] = None
    """The postal code of the address."""

    region: Optional[str] = None
    """Region or State."""

    updated_at: datetime


Addresses = Address
"""This type is deprecated and will be removed in a future release.

Please use Address instead.
"""


class Identification(BaseModel):
    id: str

    created_at: datetime

    discarded_at: Optional[datetime] = None

    expiration_date: Optional[date] = None
    """
    The date when the Identification is no longer considered valid by the issuing
    authority.
    """

    id_type: Literal[
        "ar_cuil",
        "ar_cuit",
        "br_cnpj",
        "br_cpf",
        "cl_run",
        "cl_rut",
        "co_cedulas",
        "co_nit",
        "drivers_license",
        "hn_id",
        "hn_rtn",
        "in_lei",
        "kr_brn",
        "kr_crn",
        "kr_rrn",
        "passport",
        "sa_tin",
        "sa_vat",
        "us_ein",
        "us_itin",
        "us_ssn",
        "vn_tin",
    ]
    """The type of ID number."""

    issuing_country: Optional[str] = None
    """
    The ISO 3166-1 alpha-2 country code of the country that issued the
    identification
    """

    issuing_region: Optional[str] = None
    """The region in which the identifcation was issued."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    updated_at: datetime


Identifications = Identification
"""This type is deprecated and will be removed in a future release.

Please use Identification instead.
"""


class PhoneNumber(BaseModel):
    phone_number: Optional[str] = None


PhoneNumbers = PhoneNumber
"""This type is deprecated and will be removed in a future release.

Please use PhoneNumber instead.
"""


class LegalEntity(BaseModel):
    id: str

    addresses: List[Address]
    """A list of addresses for the entity."""

    bank_settings: Optional[BankSettings] = None

    business_name: Optional[str] = None
    """The business's legal business name."""

    citizenship_country: Optional[str] = None
    """The country of citizenship for an individual."""

    compliance_details: Optional[LegalEntityComplianceDetail] = None

    created_at: datetime

    date_formed: Optional[date] = None
    """A business's formation date (YYYY-MM-DD)."""

    date_of_birth: Optional[date] = None
    """An individual's date of birth (YYYY-MM-DD)."""

    discarded_at: Optional[datetime] = None

    doing_business_as_names: List[str]

    email: Optional[str] = None
    """The entity's primary email."""

    first_name: Optional[str] = None
    """An individual's first name."""

    identifications: List[Identification]
    """A list of identifications for the legal entity."""

    industry_classifications: List[LegalEntityIndustryClassification]
    """A list of industry classifications for the legal entity."""

    last_name: Optional[str] = None
    """An individual's last name."""

    legal_entity_associations: Optional[List[LegalEntityAssociation]] = None
    """The legal entity associations and its child legal entities."""

    legal_entity_type: Literal["business", "individual", "joint"]
    """The type of legal entity."""

    legal_structure: Optional[
        Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
    ] = None
    """The business's legal structure."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    middle_name: Optional[str] = None
    """An individual's middle name."""

    object: str

    phone_numbers: List[PhoneNumber]

    politically_exposed_person: Optional[bool] = None
    """Whether the individual is a politically exposed person."""

    preferred_name: Optional[str] = None
    """An individual's preferred name."""

    prefix: Optional[str] = None
    """An individual's prefix."""

    risk_rating: Optional[Literal["low", "medium", "high"]] = None
    """The risk rating of the legal entity. One of low, medium, high."""

    suffix: Optional[str] = None
    """An individual's suffix."""

    updated_at: datetime

    wealth_and_employment_details: Optional[WealthAndEmploymentDetails] = None

    website: Optional[str] = None
    """The entity's primary website URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Document", "DocumentDetails", "DocumentDetail", "File"]


class DocumentDetail(BaseModel):
    id: str

    created_at: datetime

    discarded_at: Optional[datetime] = None

    document_identifier: str

    document_identifier_type: str

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    updated_at: datetime


DocumentDetails = DocumentDetail
"""This type is deprecated and will be removed in a future release.

Please use DocumentDetail instead.
"""


class File(BaseModel):
    content_type: Optional[str] = None
    """The MIME content type of the document."""

    filename: Optional[str] = None
    """The original filename of the document."""

    size: Optional[int] = None
    """The size of the document in bytes."""


class Document(BaseModel):
    id: str

    created_at: datetime

    discarded_at: Optional[datetime] = None

    document_details: List[DocumentDetail]

    document_type: Optional[str] = None
    """A category given to the document, can be `null`."""

    documentable_id: str
    """The unique identifier for the associated object."""

    documentable_type: Literal[
        "counterparty",
        "expected_payment",
        "external_account",
        "identification",
        "incoming_payment_detail",
        "internal_account",
        "organization",
        "paper_item",
        "payment_order",
        "transaction",
        "connection",
        "conversation",
    ]
    """The type of the associated object.

    Currently can be one of `payment_order`, `transaction`, `paper_item`,
    `expected_payment`, `counterparty`, `organization`, `case`, `internal_account`,
    `decision`, or `external_account`.
    """

    file: File

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    source: str
    """The source of the document. Can be `vendor`, `customer`, or `modern_treasury`."""

    updated_at: datetime
